@@ -1,49 +1,30 @@
-#!/usr/bin/env python3
-# coding=utf-8
-"""This module allows drawing basic graphs in the terminal."""
+"""
+termgraph.py - draw basic graphs on terminal
+https://github.com/mkaz/termgraph
+"""
 
-# termgraph.py - draw basic graphs on terminal
-# https://github.com/mkaz/termgraph
-
-from __future__ import print_function, annotations
-from typing import List, Tuple, Dict
+from __future__ import annotations
 import argparse
 import sys
-import math
 from datetime import datetime, timedelta
 from itertools import zip_longest
 from colorama import init  # type: ignore
 import os
 import re
+import importlib.metadata
 
-VERSION = "0.5.2"
+from .constants import AVAILABLE_COLORS, DAYS, DELIM, TICK, SM_TICK
+from .utils import cvt_to_readable, normalize, print_row_core
+from .data import Data
+from .args import Args
+from .chart import Chart, BarChart, StackedChart, HistogramChart
+
+__version__ = importlib.metadata.version("termgraph")
 
 init()
 
-# ANSI escape SGR Parameters color codes
-AVAILABLE_COLORS = {
-    "red": 91,
-    "blue": 94,
-    "green": 92,
-    "magenta": 95,
-    "yellow": 93,
-    "black": 90,
-    "cyan": 96,
-}
 
-DAYS = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
-UNITS = ["", "K", "M", "B", "T"]
-DELIM = ","
-TICK = "▇"
-SM_TICK = "▏"
-
-try:
-    range = xrange  # type: ignore
-except NameError:
-    pass
-
-
-def init_args() -> Dict:
+def init_args() -> dict:
     """Parse and return the arguments."""
     parser = argparse.ArgumentParser(description="draw basic graphs on terminal")
     parser.add_argument(
@@ -103,6 +84,13 @@ def init_args() -> Dict:
     parser.add_argument(
         "--version", action="store_true", help="Display version and exit"
     )
+    parser.add_argument(
+        "--no-readable", action="store_true", help="Disable the readable numbers"
+    )
+    parser.add_argument(
+        "--percentage", action="store_true", help="Display the number in percentage"
+    )
+
     if len(sys.argv) == 1:
         if sys.stdin.isatty():
             parser.print_usage()
@@ -127,7 +115,7 @@ def main():
     args = init_args()
 
     if args["version"]:
-        print("termgraph v{}".format(VERSION))
+        print(f"termgraph v{__version__}")
         sys.exit()
 
     _, labels, data, colors = read_data(args)
@@ -140,154 +128,23 @@ def main():
         pass
 
 
-def find_min(data: List):
-    """Return the minimum value in sublist of list."""
-    return min([min(sublist) for sublist in data])
-
-
-def find_max(data: List):
-    """Return the maximum value in sublist of list."""
-    return max([max(sublist) for sublist in data])
-
-
-def normalize(data: List, width: int) -> List:
-    """Normalize the data and return it."""
-
-    # We offset by the minimum if there's a negative.
-    data_offset = []
-    min_datum = find_min(data)
-    if min_datum < 0:
-        min_datum = abs(min_datum)
-        for datum in data:
-            data_offset.append([d + min_datum for d in datum])
-    else:
-        data_offset = data
-    min_datum = find_min(data_offset)
-    max_datum = find_max(data_offset)
-
-    if min_datum == max_datum:
-        return data_offset
-
-    # max_dat / width is the value for a single tick. norm_factor is the
-    # inverse of this value
-    # If you divide a number to the value of single tick, you will find how
-    # many ticks it does contain basically.
-    norm_factor = width / float(max_datum)
-    normal_data = []
-    for datum in data_offset:
-        normal_data.append([v * norm_factor for v in datum])
-
-    return normal_data
-
-
-def find_max_label_length(labels: List) -> int:
-    """Return the maximum length for the labels."""
-    return max([len(label) for label in labels])
-
-
-def cvt_to_readable(num):
-    """Return the number in a human readable format
-
-    Eg:
-    125000 -> 125.0K
-    12550 -> 12.55K
-    19561100 -> 19.561M
-    """
-
-    if num != 0:
-        neg = num < 0
-        num = abs(num)
-
-        # Find the degree of the number like if it is in thousands or millions, etc.
-        index = math.floor(math.log(num) / math.log(1000))
-
-        # Converts the number to the human readable format and returns it.
-        newNum = round(num / (1000 ** index), 3)
-        newNum *= -1 if neg else 1
-        degree = UNITS[index]
-
-    else:
-        newNum = 0
-        degree = UNITS[0]
-
-    return (newNum, degree)
-
-
-def hist_rows(data: List, args: Dict, colors: List):
-    """Prepare the Histgram graph.
-    Each row is printed through the print_row function."""
-
-    val_min = find_min(data)
-    val_max = find_max(data)
-
-    # Calculate borders
-    class_min = math.floor(val_min)
-    class_max = math.ceil(val_max)
-    class_range = class_max - class_min
-    class_width = class_range / args["bins"]
-
-    border = class_min
-    borders = []
-    max_len = len(str(border))
-
-    for b in range(args["bins"] + 1):
-        borders.append(border)
-        len_border = len(str(border))
-        if len_border > max_len:
-            max_len = len_border
-        border += class_width
-        border = round(border, 1)
-
-    # Count num of data via border
-    count_list = []
-
-    for start, end in zip(borders[:-1], borders[1:]):
-        count = 0
-        #        for d in [d]
-        for v in [row[0] for row in data]:
-            if start <= v < end:
-                count += 1
-
-        count_list.append([count])
-
-    normal_counts = normalize(count_list, args["width"])
-
-    for i, border in enumerate(zip(borders[:-1], borders[1:])):
-        if colors:
-            color = colors[0]
-        else:
-            color = None
-
-        if not args.get("no_labels"):
-            print("{:{x}} – {:{x}}: ".format(border[0], border[1], x=max_len), end="")
-
-        num_blocks = normal_counts[i]
-
-        yield (count_list[i][0], int(num_blocks[0]), 0, color)
-
-        if args.get("no_values"):
-            tail = ""
-        else:
-            tail = " {}{}".format(
-                args["format"].format(count_list[i][0]), args["suffix"]
-            )
-        print(tail)
-
 
 def horiz_rows(
-    labels: List,
-    data: List,
-    normal_dat: List,
-    args: Dict,
-    colors: List,
+    labels: list,
+    data: list,
+    normal_dat: list,
+    args: dict,
+    colors: list,
     doprint: bool = True,
 ):
     """Prepare the horizontal graph.
     Each row is printed through the print_row function."""
-    val_min = find_min(data)
+    # Create Data object to use class methods
+    data_obj = Data(data, labels)
+    val_min = data_obj.find_min()
 
     for i in range(len(labels)):
-        if args["no_labels"]:
+        if args.get("no_labels"):
             # Hide the labels.
             label = ""
         else:
@@ -295,7 +152,7 @@ def horiz_rows(
                 fmt = "{:<{x}}"
             else:
                 fmt = "{:<{x}}: "
-            label = fmt.format(labels[i], x=find_max_label_length(labels))
+            label = fmt.format(labels[i], x=data_obj.find_max_label_length())
 
         values = data[i]
         num_blocks = normal_dat[i]
@@ -314,11 +171,16 @@ def horiz_rows(
             else:
                 fmt = " {}{}{}"
 
-            if args["no_values"]:
+            if args.get("no_values"):
                 tail = args["suffix"]
             else:
-                val, deg = cvt_to_readable(values[j])
-                tail = fmt.format(args["format"].format(val), deg, args["suffix"])
+                if not args.get("no_readable"):
+                    val, deg = cvt_to_readable(values[j], args.get("percentage"))
+                    tail = fmt.format(args["format"].format(val), deg, args["suffix"])
+                else:
+                    tail = fmt.format(
+                        args["format"].format(values[j]), "", args["suffix"]
+                    )
 
             if colors:
                 color = colors[j]
@@ -346,7 +208,7 @@ def horiz_rows(
 def print_row(
     value,
     num_blocks: int,
-    val_min: int,
+    val_min: float,
     color: bool,
     label: bool = False,
     tail: bool = False,
@@ -358,71 +220,28 @@ def print_row(
     2: ▇▇▇ 3
     3: ▇▇▇▇ 4
     """
-    sys.stdout.write("\033[0m")  # no color
-    if value == 0.0:
-        sys.stdout.write("\033[90m")  # dark gray
-
     if doprint:
         print(label, tail, " ", end="")
 
-    if (num_blocks < 1 and (value > val_min or value > 0)) or (
-        doprint and value == 0.0
-    ):
-        # Print something if it's not the smallest
-        # and the normal value is less than one.
-        sys.stdout.write(SM_TICK)
-    else:
-        if color:
-            sys.stdout.write(f"\033[{color}m")  # Start to write colorized.
-        for _ in range(num_blocks):
-            sys.stdout.write(TICK)
-
-    if color:
-        sys.stdout.write("\033[0m")  # Back to original.
+    print_row_core(
+        value=value,
+        num_blocks=num_blocks,
+        val_min=val_min,
+        color=color,
+        zero_as_small_tick=doprint
+    )
 
     if doprint:
         print()
 
 
-def stacked_graph(
-    labels: List,
-    data: List,
-    normal_data: List,
-    len_categories: int,
-    args: Dict,
-    colors: List,
-):
-    """Prepare the horizontal stacked graph.
-    Each row is printed through the print_row function."""
-    val_min = find_min(data)
-
-    for i in range(len(labels)):
-        if args["no_labels"]:
-            # Hide the labels.
-            label = ""
-        else:
-            label = "{:<{x}}: ".format(labels[i], x=find_max_label_length(labels))
-
-        if args.get("space_between") and i != 0:
-            print()
-
-        print(label, end="")
-
-        values = data[i]
-        num_blocks = normal_data[i]
-
-        for j in range(len(values)):
-            print_row(values[j], int(num_blocks[j]), val_min, colors[j])
-
-        tail = " {}{}".format(args["format"].format(sum(values)), args["suffix"])
-        print(tail)
 
 
 # FIXME: globals for vertical, not ideal
 value_list, zipped_list, vertical_list, maxi = [], [], [], 0
 
 
-def vertically(value, num_blocks: int, val_min: int, color: bool, args: Dict) -> List:
+def vertically(value, num_blocks: int, val_min: int, color: bool, args: dict) -> list:
     """Prepare the vertical graph.
     The whole graph is printed through the print_vertical function."""
     global maxi, value_list
@@ -462,7 +281,7 @@ def vertically(value, num_blocks: int, val_min: int, color: bool, args: Dict) ->
     return result_list
 
 
-def print_vertical(vertical_rows: List, labels: List, color: bool, args: Dict) -> None:
+def print_vertical(vertical_rows: list, labels: list, color: bool, args: dict) -> None:
     """Print the whole vertical graph."""
     if color:
         sys.stdout.write(
@@ -486,14 +305,54 @@ def print_vertical(vertical_rows: List, labels: List, color: bool, args: Dict) -
             print("  ".join(label))
 
 
-def chart(colors: List, data: List, args: Dict, labels: List) -> None:
+def chart(colors: list, data: list, args: dict, labels: list) -> None:
     """Handle the normalization of data and the printing of the graph."""
     len_categories = len(data[0])
+    
+    # Simple bar chart case - use the BarChart class
+    if len_categories == 1 and not args["stacked"] and not args["histogram"] and not args["vertical"]:
+        # Convert CLI args dict to chart Args class, mapping incompatible keys
+        chart_args_dict = dict(args)
+        if "color" in chart_args_dict:
+            chart_args_dict["colors"] = chart_args_dict.pop("color")
+        
+        # Remove CLI-specific args that don't belong in chart Args
+        cli_only_args = ["filename", "delim", "verbose", "version"]
+        for cli_arg in cli_only_args:
+            chart_args_dict.pop(cli_arg, None)
+        
+        chart_args = Args(**chart_args_dict)
+        if colors and not chart_args.get_arg("colors"):
+            chart_args.update_args(colors=colors)
+        
+        # Create Data object and chart
+        data_obj = Data(data, labels)
+        chart_obj: Chart = BarChart(data_obj, chart_args)
+        chart_obj.draw()
+        return
+    
+    # Complex cases still use the old procedural code until we can extend chart classes
     if len_categories > 1:
         # Stacked graph
         if args["stacked"]:
-            normal_dat = normalize(data, args["width"])
-            stacked_graph(labels, data, normal_dat, len_categories, args, colors)
+            # Convert CLI args dict to chart Args class, mapping incompatible keys
+            chart_args_dict = dict(args)
+            if "color" in chart_args_dict:
+                chart_args_dict["colors"] = chart_args_dict.pop("color")
+            
+            # Remove CLI-specific args that don't belong in chart Args
+            cli_only_args = ["filename", "delim", "verbose", "version"]
+            for cli_arg in cli_only_args:
+                chart_args_dict.pop(cli_arg, None)
+            
+            chart_args = Args(**chart_args_dict)
+            if colors and not chart_args.get_arg("colors"):
+                chart_args.update_args(colors=colors)
+            
+            # Create Data object and chart
+            data_obj = Data(data, labels)
+            stacked_chart: Chart = StackedChart(data_obj, chart_args)
+            stacked_chart.draw()
             return
 
         if not colors:
@@ -528,17 +387,34 @@ def chart(colors: List, data: List, args: Dict, labels: List) -> None:
                     print_vertical(vertic, labels, colors[i], args)
 
                 print()
-                value_list.clear(), zipped_list.clear(), vertical_list.clear()
+                value_list.clear()
+                zipped_list.clear()
+                vertical_list.clear()
             return
 
     if args["histogram"]:
         if args["vertical"]:
-            print(">> Error: Vertical graph for Histogram" " is not supported yet.")
+            print(">> Error: Vertical graph for Histogram is not supported yet.")
             sys.exit(1)
 
-        for row in hist_rows(data, args, colors):
-            print_row(*row)
-            print()
+        # Convert CLI args dict to chart Args class, mapping incompatible keys
+        chart_args_dict = dict(args)
+        if "color" in chart_args_dict:
+            chart_args_dict["colors"] = chart_args_dict.pop("color")
+        
+        # Remove CLI-specific args that don't belong in chart Args
+        cli_only_args = ["filename", "delim", "verbose", "version"]
+        for cli_arg in cli_only_args:
+            chart_args_dict.pop(cli_arg, None)
+        
+        chart_args = Args(**chart_args_dict)
+        if colors and not chart_args.get_arg("colors"):
+            chart_args.update_args(colors=colors)
+        
+        # Create Data object and chart
+        data_obj = Data(data, labels)
+        hist_chart: Chart = HistogramChart(data_obj, chart_args)
+        hist_chart.draw()
         return
 
     # One category/Multiple series graph with same scale
@@ -564,8 +440,17 @@ def chart(colors: List, data: List, args: Dict, labels: List) -> None:
         print()
 
 
-def check_data(labels: List, data: List, args: Dict) -> List:
+def check_data(labels: list, data: list, args: dict) -> list:
     """Check that all data were inserted correctly. Return the colors."""
+    # Check for empty arguments
+    if not labels:
+        print(">> Error: No labels provided")
+        sys.exit(1)
+
+    if not data:
+        print(">> Error: No data provided")
+        sys.exit(1)
+
     len_categories = len(data[0])
 
     # Check that there are data for all labels.
@@ -627,7 +512,7 @@ def check_data(labels: List, data: List, args: Dict) -> List:
     return colors
 
 
-def print_categories(categories: List, colors: List) -> None:
+def print_categories(categories: list, colors: list) -> None:
     """Print a tick and the category's name for each category above
     the graph."""
     for i in range(len(categories)):
@@ -644,7 +529,7 @@ def print_categories(categories: List, colors: List) -> None:
     print("\n\n")
 
 
-def read_data(args: Dict) -> Tuple[List, List, List, List]:
+def read_data(args: dict) -> tuple[list, list, list, list]:
     """Read data from a file or stdin and returns it.
 
     Filename includes (categories), labels and data.
@@ -666,7 +551,10 @@ def read_data(args: Dict) -> Tuple[List, List, List, List]:
     if args["title"]:
         print("# " + args["title"] + "\n")
 
-    categories, labels, data, colors = ([] for _ in range(4))
+    categories: list[str] = []
+    labels: list[str | None] = []
+    data: list = []
+    colors: list = []
 
     f = None
 
@@ -676,24 +564,23 @@ def read_data(args: Dict) -> Tuple[List, List, List, List]:
             line = line.strip()
             if line:
                 if not line.startswith("#"):
-                    if line.find(DELIM) > 0:
-                        cols = line.split(DELIM)
-                    else:
-                        cols = line.split()
-
                     # Line contains categories.
                     if line.startswith("@"):
+                        cols = line.split(DELIM)
                         cols[0] = cols[0].replace("@ ", "")
                         categories = cols
 
                     # Line contains label and values.
                     else:
-                        labels.append(cols[0].strip())
-                        data_points = []
-                        for i in range(1, len(cols)):
-                            data_points.append(float(cols[i].strip()))
-
-                        data.append(data_points)
+                        if line.find(DELIM) > 0:
+                            cols = line.split(DELIM)
+                            delim = DELIM
+                        else:
+                            cols = line.split()
+                            delim = " "
+                        labeled_row = _label_row([col.strip() for col in cols], delim)
+                        data.append(labeled_row.data)
+                        labels.append(labeled_row.label)
     except FileNotFoundError:
         print(
             ">> Error: The specified file [{fname}] does not exist.".format(
@@ -717,7 +604,47 @@ def read_data(args: Dict) -> Tuple[List, List, List, List]:
     return categories, labels, data, colors
 
 
-def calendar_heatmap(data: Dict, labels: List, args: Dict) -> None:
+class _LabeledRow:
+    def __init__(self, label: str | None, data: list[float]):
+        self.label = label
+        self.data = data
+
+
+def _label_row(row: list[str], delim: str) -> _LabeledRow:
+    data = []
+    labels: list[str] = []
+    labelling = False
+
+    for text in row:
+        datum = _maybe_float(text)
+        if datum is None and not labels:
+            labels.append(text)
+            labelling = True
+        elif datum is None and labelling:
+            labels.append(text)
+        elif datum is not None:
+            data.append(datum)
+            labelling = False
+        else:
+            raise ValueError("Multiple labels not allowed: {labels}, {text}")
+
+    if labels:
+        label = delim.join(labels)
+    else:
+        label = row[0]
+        data.pop(0)
+
+    return _LabeledRow(label=label, data=data)
+
+
+def _maybe_float(text: str) -> float | None:
+    try:
+        return float(text)
+    except ValueError:
+        return None
+
+
+def calendar_heatmap(data: dict, labels: list, args: dict) -> None:
     """Print a calendar heatmap."""
     if args["color"]:
         colornum = AVAILABLE_COLORS.get(args["color"][0])
